#include "DataBases.h"

using namespace std;

/// Parse database file on initial_DataBase structure
void DataBases::GetData(string FileName){
	ifstream fichier(FileName.c_str(), ios::in); 
	vector <int> Temporary_Transaction;
	string ligne; 		  
	if (fichier){					
		while (getline(fichier, ligne)){ 
			int buffer ;
			stringstream ss(ligne);
			while(ss >> buffer){ 
				Temporary_Transaction.push_back(buffer);
			}
			ss.clear() ;				  
			initial_DataBase.push_back(Temporary_Transaction);
			Temporary_Transaction.clear(); 
		}
	}
	else{
		cerr << "ERROR in reading file" << endl;
	}
}

/// Processing database (delete no frequent items...)
void DataBases::Preprocessing_DataBase(int minsup){
	
	int total_size = 0;
	for ( unsigned int i = 0; i < initial_DataBase.size(); i++){
		for(unsigned int j = 0; j < initial_DataBase[i].size(); j++){
			total_size++;
		}
	}
	cout << "Total size database (before) = " << total_size << endl;
	
	vector <int> Items; // vector contening all items
	// Get all items of initial database
	for ( unsigned int i = 0; i < initial_DataBase.size(); i++){
		for(unsigned int j = 0; j < initial_DataBase[i].size(); j++){
			if( i == 0 ){ // First transaction => take all of its items
				Items.push_back(initial_DataBase[i][j]);
			} 
			else { // Transactions 2..|D|				
				// if the current item is not found in Items, add it
				if ( std::find(Items.begin(), Items.end(), initial_DataBase[i][j]) == Items.end() ){
					Items.push_back(initial_DataBase[i][j]);
				}
			}
		}
	}	
	sort(Items.begin(), Items.end()); // Lexicographic order on items
		
	// ---- Representation <item , set of its transactions> ----
	map < int, VECELE > Table_Verticale; // representation with integer
	vector <int> Trans_;
	// For each item give its transactions
	for(unsigned int k = 0; k < Items.size(); k++){ // for each item
		for ( unsigned int i = 0; i < initial_DataBase.size(); i++){// for each transaction
			if ( std::find(initial_DataBase[i].begin(), initial_DataBase[i].end(), Items[k])!=initial_DataBase[i].end() ){
				Trans_.push_back(i+1);
			}	
		}
		Table_Verticale[Items[k]] = Trans_; // Map with (key :item, value: cover)
		Trans_.clear();
	}
		
	// Frequency of each item
	vector <int> Frequencies;
	map<int, VECELE>::iterator itv; // iterator
	for(itv=Table_Verticale.begin();itv!=Table_Verticale.end(); ++itv){
		Frequencies.push_back(itv->second.size());		
	}		
	
	// Construction of vector V where each item is encoded with its rank according to frequence
	vector< pair<int, int> > V;
	for (unsigned int i =0; i < Items.size(); i++){
		V.push_back(make_pair(Frequencies[i], Items[i]));
	}sort(V.rbegin(), V.rend());
	
	// Get rank of each item
	for(unsigned int i =0; i < V.size(); i++){
		Code.push_back(V[i].second);
	}

	// New datbase with the encoding
	vector <int> temp_traduit;
	for ( unsigned int i = 0; i < initial_DataBase.size(); i++){
		for ( unsigned int j =0; j < initial_DataBase[i].size(); j++){
			for (unsigned int k = 0; k < Code.size(); k++){	
				if (Code[k] == initial_DataBase[i][j]){
					temp_traduit.push_back(k+1); // replace item with its rank
				}
			}
		}
		Processed_DataBase.push_back(temp_traduit);
		temp_traduit.clear();
	}
	
	// keep only frequent items		
	Code.clear();
	for(unsigned int i =0; i < V.size(); i++){
		if ( V[i].first >= minsup )
			Code.push_back(V[i].second);
		else
			break;
	}
		
	// New database without no frequent items 
	for ( unsigned int i = 0; i < Processed_DataBase.size(); i++){
		for ( unsigned int j =0; j < Processed_DataBase[i].size(); j++){
			if (Processed_DataBase[i][j] > Code.size() ){
				Processed_DataBase[i].erase(std::remove(Processed_DataBase[i].begin(), Processed_DataBase[i].end(), Processed_DataBase[i][j]), Processed_DataBase[i].end());
				j--; // L'element suivant recule d'un pas, il faut le tester lui aussi
			}		
		}sort(Processed_DataBase[i].begin(), Processed_DataBase[i].end());
	}	
}

/// Alimenter matrice booléene 
void DataBases::Create_Boolean_Matrix(){
	
	int total_size = 0;
	for ( unsigned int i = 0; i < Processed_DataBase.size(); i++){
		for(unsigned int j = 0; j < Processed_DataBase[i].size(); j++){
			total_size++;
		}
	}
	cout << "Total size database (after) = " << total_size << endl;
	
	vector <int> Tmp_Boolean;
	for ( unsigned int i = 0; i < Processed_DataBase.size(); i++){ // create a matrix of (n X m) "0"
		for(int u =0; u < Code.size(); u++)
			Tmp_Boolean.push_back(0);				
		Boolean_Base.push_back(Tmp_Boolean);			
		Tmp_Boolean.clear();			
	}		
	// Mettre à "1" les cases qui correspondent à un item dans la base initiale 
	for ( unsigned int i = 0; i < Processed_DataBase.size(); i++){	
		for ( unsigned int j =0; j < Processed_DataBase[i].size(); j++){
			Boolean_Base[i][Processed_DataBase[i][j]-1] = 1;
		}
	}	
}	

/// Create vertical database with bitset
void DataBases::Create_Vertical_DataBase(){	
	int trs0 = Boolean_Base.size(); // nbr transactions
	for(int i = 0; i < Boolean_Base[0].size(); i++){ // for each item
		boost::dynamic_bitset<> Vertical_bitset(trs0); // a bitset of size |T|
		for(int j = 0; j < Boolean_Base.size(); j++){
			if (Boolean_Base[j][i] == 1) 
				Vertical_bitset[j] = 1;
			else 
				Vertical_bitset[j] = 0;
		}
		Vertical_DataBase.push_back(Vertical_bitset); // add the bitset to structure "vector of bitset"
	}
}

/// Create horizontal database with bitset 
void DataBases::Create_Horizontal_DataBase(){
	int itm0 = Code.size(); // nbr items
	for(int i = 0; i < Boolean_Base.size(); i++){ // for each transaction
		boost::dynamic_bitset<> Horizontal_bitset(itm0); // a bitset of size |I|
		for(int j =0; j < Boolean_Base[i].size(); j++){
			if(Boolean_Base[i][j] == 1)
				Horizontal_bitset[j] = 1;
			else
				Horizontal_bitset[j] = 0;
		}
		Horizontal_DataBase.push_back(Horizontal_bitset); // add the bitset to structure "vector of bitset"
	}
}

